// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include <QMessageBox>

#ifdef Q_OS_ANDROID
#include <android/log.h>
#endif

class Logger : public QObject
{
    Q_OBJECT
public:
    Logger(const Logger &) = delete;
    Logger &operator=(const Logger &) = delete;

    static Logger &instance()
    {
        static Logger m_instance;
        return m_instance;
    }

    void log(QtMsgType type, const QMessageLogContext &context, const QString &msg)
    {
        QString logPrefix, logSuffix, newLog;

        QByteArray localMsg = msg.toLocal8Bit();
        const char *file = context.file ? context.file : "";
        const char *function = context.function ? context.function : "";

        switch (type) {
        case QtDebugMsg:
            logPrefix = QStringLiteral("Debug: ");
            break;
        case QtInfoMsg:
            logPrefix = QStringLiteral("Info: ");
            break;
        case QtWarningMsg:
            logPrefix = QStringLiteral("Warning: ");
            break;
        case QtCriticalMsg:
            logPrefix = QStringLiteral("Critical: ");
            break;
        case QtFatalMsg:
            logPrefix = QStringLiteral("Fatal: ");
            logSuffix = QStringLiteral(" (%1:%2, %3)").arg(file).arg(context.line).arg(function);
            break;
        }

        newLog += logPrefix + localMsg + logSuffix + "\n";
#ifdef Q_OS_ANDROID
        __android_log_print(ANDROID_LOG_DEBUG, "Qt_UI_Viewer", "%s", qPrintable(newLog));
#else
        fprintf(stderr, "%s", qPrintable(newLog));
#endif

        if (type == QtCriticalMsg || type == QtFatalMsg) {
            QMessageBox msgBox{QMessageBox::Critical, "Critical:", msg, QMessageBox::Ok};
            msgBox.exec();
        }

        emit logMessage(type, newLog);
    }

private:
    Logger()
    {
        qInstallMessageHandler(
            [](QtMsgType type, const QMessageLogContext &context, const QString &msg) {
                Logger::instance().log(type, context, msg);
            });
    }

signals:
    void logMessage(QtMsgType type, QString &msg);
};
